from collections import deque, defaultdict

def find_guarded_warehouses(n, m, k, roads, guards):
    # Создаем граф в виде списка смежности
    graph = defaultdict(list)
    
    for a, b in roads:
        graph[a].append(b)
        graph[b].append(a)
    
    # Множество для хранения охраняемых складов
    guarded = set()
    
    # Обрабатываем каждого охранника
    for p, h in guards:
        # Используем BFS для поиска охраняемых складов
        queue = deque([(p, 0)])  # (текущий склад, текущее расстояние)
        visited = set()  # Множество посещенных складов
        
        while queue:
            current, distance = queue.popleft()
            if current in visited:
                continue
            
            visited.add(current)
            guarded.add(current)  # Добавляем склад в охраняемые
            
            if distance < h:  # Если не превышаем выносливость охранника
                for neighbor in graph[current]:
                    if neighbor not in visited:
                        queue.append((neighbor, distance + 1))
    
    # Преобразуем охраняемые склады в отсортированный список
    guarded_list = sorted(guarded)
    
    return len(guarded_list), guarded_list

# Чтение входных данных
n, m, k = map(int, input().strip().split())
roads = [tuple(map(int, input().strip().split())) for _ in range(m)]
guards = [tuple(map(int, input().strip().split())) for _ in range(k)]

# Поиск охраняемых складов
count, guarded_warehouses = find_guarded_warehouses(n, m, k, roads, guards)

# Вывод результатов
print(count)
if count > 0:
    print(' '.join(map(str, guarded_warehouses)))
else:
    print()
